﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using RS232Port;

namespace GpsModule {
	public partial class GpsModuleMainForm : Form {

		GpsStrategy _strategy;

		bool _silent = true;
		public bool Silent {
			get { return _silent; }
			set {
				_silent = value;
				if (_silent) {
					_strategy.updateUiEvent -= updateUi;
					_strategy.setSilentModeEvent -= setSilentMode;
				} else {
					_strategy.updateUiEvent += updateUi;
					_strategy.setSilentModeEvent += setSilentMode;
				}
			}
		}

		//called by delegate
		private void setSilentMode() {
			if (Silent != _strategy.Silent)
				Silent = _strategy.Silent;
		}

		public GpsModuleMainForm() {
			InitializeComponent();
			_strategy = new GpsStrategy("COM6");
			initializeObject();
		}

		public GpsModuleMainForm(ref GpsStrategy strategy) {
			InitializeComponent();
			_strategy = strategy;
			initializeObject();
		}

		private void initializeObject() {
			if (_strategy.Status.Contains("opened")) {
				toolStripStatusLabel1.Text = "gps_init";
				toolStripStatusLabel1.BackColor = Color.Green;
			}
			toolStripStatusLabel2.Text = _strategy.Status;
		}

		private void updateUi() {
			debugTextBox.Text = _strategy.Gga.Data.Sentance + "\r\n" + _strategy.Rmc.Data.Sentance + "\r\n";
			timeTextBox.Text = _strategy.Gga.Data.Time.ToString();
			latTextBox.Text = _strategy.Gga.Data.Latitude.ToString();
			nsTextBox.Text = _strategy.Gga.Data.NorthOrSouth.ToString();
			lonTextBox.Text = _strategy.Gga.Data.Longitude.ToString();
			ewTextBox.Text = _strategy.Gga.Data.EastOrWest.ToString();
			qualityTextBox.Text = _strategy.Gga.Data.QualityIndicator.ToString();
			satelitesTextBox.Text = _strategy.Gga.Data.SatelitesInView.ToString();
			dilutionTextBox.Text = _strategy.Gga.Data.HorizontalDilutionOfPrecision.ToString();
			altitudeTextBox.Text = _strategy.Gga.Data.AntennaAltitude.ToString();
			aUnitsTextBox.Text = _strategy.Gga.Data.UnitsOfAntennaAltitude.ToString();
			geoidTextBox.Text = _strategy.Gga.Data.GeoidalSeparation.ToString();
			gUnitsTextBox.Text = _strategy.Gga.Data.UnitsOfGeoidalSeperation.ToString();
			speedTextBox.Text = _strategy.Rmc.Data.SpeedOverGround.ToString();
			trackTextBox.Text = _strategy.Rmc.Data.TrackMadeGood.ToString();
			magTextBox.Text = _strategy.Rmc.Data.MagneticVariation.ToString();
		}

		private void rS232PortToolStripMenuItem_Click(object sender, EventArgs e) {
			_strategy.showSerialPort();
		}

		private void silentCheckBox_CheckedChanged(object sender, EventArgs e) {
			_strategy.setSilentMode(silentCheckBox.Checked);
			Silent = silentCheckBox.Checked;
		}

		private void hideButton_Click(object sender, EventArgs e) {
			this.Hide();
		}

		private void clearDebugWindowButton_Click(object sender, EventArgs e) {
			debugTextBox.Clear();
		}

		private void openComButton_Click(object sender, EventArgs e) {
			toolStripStatusLabel2.Text = _strategy.openPort();
			if (_strategy.Status.Contains("opened")) {
				toolStripStatusLabel1.BackColor = Color.Green;
				toolStripStatusLabel1.Text = "gps_init";
			}
		}

		private void closeComButton_Click(object sender, EventArgs e) {
			toolStripStatusLabel2.Text = _strategy.closePort();
			toolStripStatusLabel1.BackColor = Color.Red;
			toolStripStatusLabel1.Text = "not initialized";
		}
	}
}
