﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace RacePlotter {
	public partial class RacePlotterForm : Form {
		bool _addFiducials = false;
		bool _addWaypoints = false;
		bool _editFiducials = false;
		bool _editWaypoints = false;
		RacePlotterStrategy _strategy;
		PointSelector _pointPicker;
		PointEditor _pointEditor;
		public RacePlotterForm() {
			InitializeComponent();
			_strategy = new RacePlotterStrategy(".\\maps\\videra\\");
			raceMapPictureBox.Image = _strategy.RaceMap;
			redrawMap();
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
			this.Close();
		}

		private void redrawMap() {
			_strategy.redrawMap();
			raceMapPictureBox.Image = _strategy.RaceMap;
			raceMapPictureBox.Invalidate();
		}

		private void raceMapPictureBox_MouseClick(object sender, MouseEventArgs e) {
			if (_addFiducials) {
				Bitmap b = new Bitmap(raceMapPictureBox.Image);
				_strategy.drawClickedWaypoint(ref b, (int)(e.X * ((double)(b.Width)
					/ (double)(raceMapPictureBox.Width)))
					, (int)(e.Y * ((double)(b.Height) / (double)(raceMapPictureBox.Height))));
				raceMapPictureBox.Image = b;
				raceMapPictureBox.Invalidate();
				_pointPicker = new PointSelector(_strategy.Fiducials);
				_pointPicker.pointSelectEvent += onFiducialtAddSelect;
				//_pointPicker.waypointSelectCancelEvent += onWaypointSelectCancel;
				_pointPicker.Show();
			} else if(_addWaypoints) {
				_pointEditor = new PointEditor(getUtmPointFromScreen(e));
				_pointEditor.pointEditEvent += onWaypointAdd;
				_pointEditor.Show();
			} else if (_editWaypoints) {

			}
		}


		private void onWaypointAdd() {
			_strategy.Waypoints.Add(_pointEditor.getEditedPoint());
			_pointEditor.Close();
			redrawMap();
		}

		private void onWaypointEditSelect() {
			_pointEditor = new PointEditor(_pointPicker.getSelectedPoint());
			_pointEditor.pointEditEvent += onWaypointEdit;
			_pointEditor.Show();
		}

		private void onWaypointEdit() {
			redrawMap();
		}

		private void onFiducialtAddSelect() {
			if (_strategy.Fiducial1.Handle == "")
				_strategy.Fiducial1 = _pointPicker.getSelectedPoint();
			else
				_strategy.Fiducial2 = _pointPicker.getSelectedPoint();
			if (_strategy.Fiducial2.Handle != "") {
				_strategy.setScale();
				raceMapPictureBox.Image = _strategy.RaceMap;
				raceMapPictureBox.Invalidate();
				_addFiducials = false;
				fiducialsToolStripMenuItem.Enabled = true;
			}
		}

		private void onWaypointSelectCancel() {
		}

		private UtmConvert.UtmPoint getUtmPointFromScreen(MouseEventArgs e) {
			double mouseX = (double)e.X * ((double)raceMapPictureBox.Image.Width
				/ (double)raceMapPictureBox.Width);
			double mouseY = (double)e.Y * ((double)raceMapPictureBox.Image.Height
				/ (double)raceMapPictureBox.Height);
			double x = (double)_strategy.Click1.X / _strategy.ScaleX + (mouseX
				- (double)_strategy.Click1.X) / _strategy.ScaleDeltaX;
			double y = (double)_strategy.Click1.Y / _strategy.ScaleY + (mouseY
				- (double)_strategy.Click1.Y) / _strategy.ScaleDeltaY;
			return new UtmConvert.UtmPoint(new UtmConvert.Point(x, y), "", "", ""); 

		}

		private void raceMapPictureBox_MouseMove(object sender, MouseEventArgs e) {
			if (_strategy.Fiducial2.Handle != "" && RacePlotterForm.ActiveForm != null) {
				toolStripStatusLabel1.Text = Math.Round(getUtmPointFromScreen(e).Point.X, 4).ToString()
					+ ", " + Math.Round(getUtmPointFromScreen(e).Point.Y, 4).ToString();
			}
		}

		private void addToolStripMenuItem1_Click(object sender, EventArgs e) {
			_addFiducials = true;
			fiducialsToolStripMenuItem.Enabled = false;
			_strategy.resetFiducial();
		}

		private void addToolStripMenuItem2_Click(object sender, EventArgs e) {
			_addFiducials = false;
			fiducialsToolStripMenuItem.Enabled = false;
			_addWaypoints = true;
		}

		private void saveToolStripMenuItem_Click(object sender, EventArgs e) {
			_strategy.saveWaypoints();
		}

		private void editToolStripMenuItem2_Click(object sender, EventArgs e) {
			_pointPicker = new PointSelector(_strategy.Waypoints);
			_pointPicker.pointSelectEvent += onWaypointEditSelect;
			_pointPicker.Show();
		}

	}
}
