﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UtmConvert;
using System.Drawing;
using System.IO;

namespace RacePlotter {
	class RacePlotterStrategy : RacePlotterData {

		public RacePlotterStrategy(string racePath) {
			_racePath = racePath;
			InitializationParameters.readIniParameters(_racePath);
			_raceMap = new Bitmap(_racePath + "map.jpg");
			_click1 = new System.Drawing.Point();
			_click2 = new System.Drawing.Point();
			_statePosition = new UtmPoint(new UtmConvert.Point
				(488816.66, 4874829.39), "10T", "state", "state position");
			_gpsPosition = new UtmPoint(new UtmConvert.Point
				(488815.8, 4874822.61), "10T", "gps", "gps position");
			_fiducial1 = new UtmPoint();
			_fiducial2 = new UtmPoint();
			_fiducials = new List<UtmPoint>();
			Waypoints = new List<UtmPoint>();
			parseFiducialList();
			if (InitializationParameters.IniFileParameters.Count == 4) {
				_scaleDeltaX = double.Parse(InitializationParameters.IniFileParameters["scaleDeltaX"]);
				_scaleDeltaY = double.Parse(InitializationParameters.IniFileParameters["scaleDeltaY"]);
				_scaleX = double.Parse(InitializationParameters.IniFileParameters["scaleX"]);
				_scaleY = double.Parse(InitializationParameters.IniFileParameters["scaleY"]);
				_fiducial1 = _fiducials[0];
				_fiducial2 = _fiducials[1];
				drawFiducials();
			}
			parseWaypointList();
		}

		public void redrawMap() {
			drawFiducials();
			drawRacePath(ref _raceMap);
			drawRobotPosition(ref _raceMap, _gpsPosition);
			drawRobotPosition(ref _raceMap, _statePosition);
		}

		private void drawGpsLocation() {

		}

		//load fiducials into fiducial list
		private void parseFiducialList() {
			if (File.Exists(_racePath + "fiducials.xml")) {
				XmlParser x = new XmlParser();
				x.readFile(_racePath + "fiducials.xml");
				x.Parse("ddd_dd");
				_fiducials  = x.UtmPointSetList[0].Points;
			}
		}

		//load waypoints into waypoint list
		private void parseWaypointList() {
			if (File.Exists(_racePath + "waypoints.xml")) {
				XmlParser x = new XmlParser();
				x.readFile(_racePath + "waypoints.xml");
				x.Parse("ddd_dd");
				_waypoints = x.UtmPointSetList[0].Points;
			}
		}


		public void saveWaypoints() {
			List<UtmPointSet> s = new List<UtmPointSet>();
			UtmPointSet ps = new UtmPointSet(_racePath + "waypoints"
				, _waypoints, "NAD83/WGS84");
			s.Add(ps);
			Printer p = new Printer();
			p.writeUtmXmlFile(s);
		}

		//public bool pointMatch(UtmPoint p) {
		//    if (_point2 == "")
		//        return _point1 == p.Handle;
		//    else return _point2 == p.Handle;
		//}

		private void drawFiducials() {
			_raceMap = new Bitmap(_racePath + "map.jpg");
			Graphics g = Graphics.FromImage(_raceMap);
			Pen p = new Pen(Color.Red, 1);
			float x = (float)(_fiducials[0].Point.X * _scaleX);
			float y = (float)(_fiducials[0].Point.Y * _scaleY);
			_click1 = new System.Drawing.Point((int)x, (int)y);
			g.DrawLine(p, x - 7, y, x + 7, y);
			g.DrawLine(p, x, y - 7, x, y + 7);
			x += (float)((_fiducials[1].Point.X - _fiducials[0].Point.X) * _scaleDeltaX);
			y += (float)((_fiducials[1].Point.Y - _fiducials[0].Point.Y) * _scaleDeltaY);
			_click2 = new System.Drawing.Point((int)x, (int)y);
			g.DrawLine(p, x - 7, y, x + 7, y);
			g.DrawLine(p, x, y - 7, x, y + 7);
		}

		public void drawRacePath(ref Bitmap map) {
			Graphics g = Graphics.FromImage(map);
			Pen pen = new Pen(Color.Red, 2);
			SolidBrush brush = new SolidBrush(Color.Orange);
			SolidBrush gBrush = new SolidBrush(Color.Gray);
			float nextX = 0, nextY = 0, deltaX = 0, deltaY = 0, x1 = 0, y1 = 0;
			for (int i = 0; i < Waypoints.Count; ++i) {
				deltaX = (float)(Waypoints[i].Point.X - _fiducial1.Point.X);
				deltaY = (float)(Waypoints[i].Point.Y - _fiducial1.Point.Y);
				x1 = (float)Waypoints[i].Point.X;
				y1 = (float)Waypoints[i].Point.Y;
				if (Waypoints[i].Note == "dummy")
					g.FillEllipse(gBrush, x1 * (float)_scaleX + deltaX * (float)_scaleDeltaX
						- 7, y1 * (float)_scaleY + deltaY * (float)_scaleDeltaY - 7, 15, 15);
				else
					g.FillEllipse(brush, x1 * (float)_scaleX + deltaX * (float)_scaleDeltaX
						- 7, y1 * (float)_scaleY + deltaY * (float)_scaleDeltaY - 7, 15, 15);
				if (i > 0) {
					g.DrawLine(pen, x1 * (float)_scaleX + nextX, y1 * (float)_scaleY + nextY
						, x1 * (float)_scaleX + deltaX * (float)_scaleDeltaX
						, y1 * (float)_scaleY + deltaY * (float)_scaleDeltaY);
				}
				nextX = (float)_scaleX + deltaX * (float)_scaleDeltaX;
				nextY = (float)_scaleY + deltaY * (float)_scaleDeltaY;
			}
		}

		public void drawRobotPosition(ref Bitmap map, UtmConvert.UtmPoint p) {
			Graphics g = Graphics.FromImage(map);
			Pen pen = new Pen(Color.Red, 2);
			SolidBrush hBrush = new SolidBrush(Color.HotPink);
			SolidBrush yBrush = new SolidBrush(Color.Yellow);
			float deltaX = 0, deltaY = 0, x1 = 0, y1 = 0;
			deltaX = (float)(p.Point.X - _fiducial1.Point.X);
			deltaY = (float)(p.Point.Y - _fiducial1.Point.Y);
			x1 = (float)p.Point.X;
			y1 = (float)p.Point.Y;
			if (p.Note == "gps")
				g.FillEllipse(hBrush, x1 * (float)_scaleX + deltaX * (float)_scaleDeltaX
					- 7, y1 * (float)_scaleY + deltaY * (float)_scaleDeltaY - 7, 15, 15);
			else
				g.FillEllipse(yBrush, x1 * (float)_scaleX + deltaX * (float)_scaleDeltaX
					- 7, y1 * (float)_scaleY + deltaY * (float)_scaleDeltaY - 7, 15, 15);
		}


		public void resetFiducial() {
			_fiducial1 = new UtmPoint();
			_fiducial2 = new UtmPoint();
			_click1 = new System.Drawing.Point();
			_click2 = new System.Drawing.Point();
		}

		public void drawClickedWaypoint(ref Bitmap map, int x, int y) {
			if (_click1.IsEmpty) {
				_click1.X = x;
				_click1.Y = y;
			} else {
				_click2.X = x;
				_click2.Y = y;
			} 
			Graphics g = Graphics.FromImage(map);
			SolidBrush brush = new SolidBrush(Color.Green);
			g.FillEllipse(brush, x - 7, y - 7, 15, 15);
		}

		public void setScale() {
			double deltaClickX = (double)Click2.X - (double)Click1.X;
			double deltaClickY = (double)Click2.Y - (double)Click1.Y;
			double deltaUtmX = (double)(Fiducial2.Point.X - Fiducial1.Point.X);
			double deltaUtmY = (double)(Fiducial2.Point.Y - Fiducial1.Point.Y);
			_scaleDeltaX = deltaClickX / deltaUtmX;
			_scaleDeltaY = deltaClickY / deltaUtmY;
			_scaleX = (double)Click1.X / (double)Fiducial1.Point.X;
			_scaleY = (double)Click1.Y / (double)Fiducial1.Point.Y;
			writeIniFile();
			drawFiducials();
		}

		private void writeIniFile() {
			StreamWriter s = new StreamWriter(_racePath + "map.ini");
			s.Write(
				"scaleDeltaX=" + _scaleDeltaX + "\r\n"
				+ "scaleDeltaY=" + _scaleDeltaY+ "\r\n"
				+ "scaleX=" + _scaleX+ "\r\n"
				+ "scaleY=" + _scaleY + "\r\n"
				);
			s.Close();
		}

	}
}
