﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using GpsModule;
using NavigationController;
using PlatformController;
using VisionSystem;
using UtmConvert;

namespace system_controller {
    public partial class SystemControllerMainForm : Form {

		public delegate void updateStatusEventHandler();
		public event updateStatusEventHandler updateResetEvent;

		SystemData _data;
		RaceStrategy _strategy;
        PointPickerForm _waypointChooser;
        WaypointInserter _waypointInserter;
		bool _silent = true;

		public bool Silent {
			get { return _silent; }
			set {
				_silent = value;
				if (_silent) {
					_data.Navigation.updateDataEvent -= updateNavUi;
					_data.Platform.updateDataEvent -= updatePlatUi;
					_data.Gps.updateDataEvent -= updateGpsUi;
					_strategy.updateUiEvent -= updateStratUi;
				} else {
					_data.Navigation.updateDataEvent += updateNavUi;
					_data.Platform.updateDataEvent += updatePlatUi;
					_data.Gps.updateDataEvent += updateGpsUi;
					_strategy.updateUiEvent += updateStratUi;
				}
			}
		}
		Bitmap backupMap;

        public SystemControllerMainForm() {
            InitializeComponent();
			newRace();
			_data.Navigation.updateResetEvent += newRace;
			updateResetEvent += newRace;
        }

		private void silentCheckBox_CheckedChanged(object sender, EventArgs e) {
			_data.Platform.setSilentMode(silentCheckBox.Checked);
			_data.Navigation.setSilentMode(silentCheckBox.Checked);
			_data.Gps.setSilentMode(silentCheckBox.Checked);
			Silent = silentCheckBox.Checked;
		}

		private void updateNavUi() {
			//navigation Data
            xAccTextBox.Text = Math.Round(_data.Navigation.Data.XAccAverage, 2).ToString();
            yAccTextBox.Text = Math.Round(_data.Navigation.Data.YAccAverage,2).ToString();
            zAccTextBox.Text = Math.Round(_data.Navigation.Data.ZAccAverage,2).ToString();
            leftIrTextBox.Text = Math.Round(_data.Navigation.Data.LeftIrAverage,2).ToString();
            rightIrTextBox.Text = Math.Round(_data.Navigation.Data.RightIrAverage,2).ToString();
            compassTextBox.Text = Math.Round(_data.Navigation.Data.CompassHeading,2).ToString();
            systemVoltageTextBox.Text = Math.Round(_data.Navigation.Data.SystemBatteryAverage,2).ToString();
            platformVoltageTextBox.Text = Math.Round(_data.Navigation.Data.PlatformBatteryAverage,2).ToString();
		}
		private void updatePlatUi() {
			//platform data
            encoderCountsTextBox.Text = _data.Platform.Data.EncoderCounts.ToString();
			textBox3.Text = Math.Round(_data.Platform.Data.Meters, 2).ToString();
			leftBumperTextBox.Text = _data.Platform.Data.LeftBumper.ToString();
			rightBumperTextBox.Text = _data.Platform.Data.RightBumper.ToString();
			leftSonarTextBox.Text = Math.Round(_data.Platform.Data.LeftSonar, 2).ToString();
			centerSonarTextBox.Text = Math.Round(_data.Platform.Data.CenterSonar, 2).ToString();
			rightSonarTextBox.Text = Math.Round(_data.Platform.Data.RightSonar, 2).ToString();
			throttlePercentTextBox.Text = _data.Platform.Data.Throttle.ToString();
			steeringPercentTextBox.Text = _data.Platform.Data.Steering.ToString();
			reversingTextBox.Text = _data.Platform.Data.Reversing.ToString();
		}
		private void updateGpsUi() {
			//gps data
			gpsFixTextBox.Text = _data.Gps.Gga.Data.QualityIndicator.ToString();
			if (_data.Gps.Gga.Data.QualityIndicator > 0) {
				latTextBox.Text = _data.Gps.Gga.Data.Latitude;
				lonTextBox.Text = _data.Gps.Gga.Data.Longitude;
				_data.ConvertUtm.convertLatLonToUtm(ConvertDegRad.getRadians(
					_data.Gps.Gga.Data.LatDeg, _data.Gps.Gga.Data.LatMin
					, _data.Gps.Gga.Data.NorthOrSouth)
					, ConvertDegRad.getRadians(_data.Gps.Gga.Data.LonDeg
					, _data.Gps.Gga.Data.LonMin, _data.Gps.Gga.Data.EastOrWest));
				gpsNorthingTextBox.Text = Math.Round(_data.ConvertUtm.Northing, 2).ToString();
				gpsEastingTextBox.Text = Math.Round(_data.ConvertUtm.Easting, 2).ToString();

				if (_data.UtmPoint2.Handle != "") {
					//draw robot gps position
					Bitmap b = new Bitmap(backupMap);
					_data.drawCurrentPosition(ref b, new UtmPoint(new UtmConvert.Point(
						_data.ConvertUtm.Easting,_data.ConvertUtm.Northing), "", "", ""));
					mapPictureBox.Image = b;
					mapPictureBox.Invalidate();
					//mapPictureBox.Image = b2;
				}
			}
		}
		private void updateStratUi() {
			//strategy data
			eastingTextBox.Text = Math.Round(_strategy.CurrentVectorLocation.Point.X, 2).ToString();
			northingTextBox.Text = Math.Round(_strategy.CurrentVectorLocation.Point.Y, 2).ToString();
			waypointDistanceTextBox.Text = Math.Round(_strategy.WaypointDistanceVector, 2).ToString();
			thetaTextBox.Text = Math.Round(_strategy.ThetaVector, 2).ToString();
			crossTrackErrorTextBox.Text = Math.Round(_strategy.CrossTrackErrorVector, 2).ToString();
			turnDirectionTextBox.Text = _strategy.TurnDirection;
			offsetCompassTextBox.Text = Math.Round(_strategy.OffsetCompassHeading, 2).ToString();

			if (_data.UtmPoint2.Handle != "") {
				//draw robot position
				Bitmap b = new Bitmap(backupMap);
				_data.drawCurrentPosition(ref b, _strategy.CurrentVectorLocation);
				mapPictureBox.Image = b;
				mapPictureBox.Invalidate();
				//mapPictureBox.Image = b2;
			}
		}

        private void exitToolStripMenuItem1_Click(object sender, EventArgs e) {
			_data.Navigation.writeToNavigationController(NavigationController
				.NavigationCommands.cmdArray[(int)NavigationController
				.NavigationCommands.commands.resetEncoderLedOff]);
			_data.safeShutdown();
            this.Close();
        }

        private void platformToolStripMenuItem1_Click(object sender, EventArgs e) {
            _data.showPlatform();
        }

        private void navigationToolStripMenuItem1_Click(object sender, EventArgs e) {
            _data.showNavigation();
        }

        private void visionToolStripMenuItem1_Click(object sender, EventArgs e) {
            _data.showVision();
        }

        private void gPSToolStripMenuItem2_Click(object sender, EventArgs e) {
            _data.showGps();
        }

        private void uTMConvertToolStripMenuItem1_Click(object sender, EventArgs e) {
            _data.showUtm();
        }

        private void mapPictureBox_MouseDoubleClick(object sender, MouseEventArgs e) {
            if (_data.UtmPoint2.Handle != "") {
                double mouseX = (double)e.X * ((double)(mapPictureBox.Image.Width)
                    / (double)(mapPictureBox.Width));
                double mouseY = (double)e.Y * ((double)(mapPictureBox.Image.Height)
                    / (double)(mapPictureBox.Height));
                double x = (double)(_data.Click1.X) / (double)(_data.ScaleX)
                    + (mouseX - (double)_data.Click1.X) / (double)(_data.ScaleDeltaX);
                double y = (double)(_data.Click1.Y) / (double)(_data.ScaleY)
                    + (mouseY - (double)_data.Click1.Y) / (double)(_data.ScaleDeltaY);
                UtmPoint uPt = new UtmPoint();
                uPt.Point.X = x;
                uPt.Point.Y = y;
                uPt.Zone = _data.Parser.UtmPointSetList[0].Points[0].Zone;
                _waypointInserter = new WaypointInserter(uPt
                    , _data.Parser.UtmPointSetList[0].Points.Count);
                _waypointInserter.waypointInsertEvent += onWaypointInsert;
                _waypointInserter.Show();
            }
        }

        private void onWaypointSelect() {
            if (_data.UtmPoint1.Handle == "")
                _data.UtmPoint1 = _waypointChooser.getSelectedPoint();
            else
                _data.UtmPoint2 = _waypointChooser.getSelectedPoint();
            if (_data.UtmPoint1.Handle == _data.UtmPoint2.Handle)
                onWaypointSelectCancel();
            if (_data.UtmPoint2.Handle != "") {
                Bitmap b = new Bitmap(mapPictureBox.Image);
                _data.drawRedLines(ref b);
                mapPictureBox.Image = b;
				backupMap = new Bitmap(mapPictureBox.Image);
                mapPictureBox.Invalidate();
            }
        }

        private void onWaypointSelectCancel() {
            _data.Parser.Parse("ddd_dd");
            _data.Click1 = new System.Drawing.Point();
            _data.Click2 = new System.Drawing.Point();
            _data.UtmPoint1 = new UtmPoint();
            _data.UtmPoint2 = new UtmPoint();
            mapPictureBox.Image = new Bitmap(_data.MapFileName);
        }

        private void mapPictureBox_MouseMove(object sender, MouseEventArgs e) {
            if (_data.UtmPoint2.Handle != "") {
                int mouseX = (int)((float)e.X * ((float)(mapPictureBox.Image.Width)
                    / (float)(mapPictureBox.Width)));
                int mouseY = (int)((float)e.Y * ((float)(mapPictureBox.Image.Height)
                    / (float)(mapPictureBox.Height)));
                int x = (int)((float)(_data.Click1.X) / _data.ScaleX + (float)(mouseX
                    - _data.Click1.X) / _data.ScaleDeltaX);
                int y = (int)((float)(_data.Click1.Y) / _data.ScaleY + (float)(mouseY
                    - _data.Click1.Y) / _data.ScaleDeltaY);
                if (SystemControllerMainForm.ActiveForm != null)
                    toolTip1.Show(x.ToString() + ", " + y.ToString()
                        , SystemControllerMainForm.ActiveForm
                    , e.X + mapGroupBox.Location.X, e.Y + mapGroupBox.Location.Y);
            }
        }

        private void mapPictureBox_MouseClick(object sender, MouseEventArgs e) {
            if (_data.UtmPoint2.Handle == "") {
                Bitmap b = new Bitmap(mapPictureBox.Image);
                _data.drawClickedWaypoint(ref b, (int)(e.X * ((double)(b.Width)
                    / (double)(mapPictureBox.Width)))
                    , (int)(e.Y * ((double)(b.Height) / (double)(mapPictureBox.Height))));
                mapPictureBox.Image = b;
                mapPictureBox.Invalidate();
                _waypointChooser = new PointPickerForm(_data.Parser.UtmPointSetList);
                _waypointChooser.waypointSelectEvent += onWaypointSelect;
                _waypointChooser.waypointSelectCancelEvent += onWaypointSelectCancel;
                _waypointChooser.Show();
            }
        }

        private void onWaypointInsert() {
            _data.Parser.UtmPointSetList[0].Points.Insert(
                _waypointInserter.getInsertedWaypointIndex()
                , _waypointInserter.getInsertedWaypoint());
            Bitmap b = new Bitmap(_data.MapFileName);
            _data.drawRedLines(ref b);
            mapPictureBox.Image = b;
            mapPictureBox.Invalidate();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e) {
			newRace();
        }

		public void newRace() {
			try {
				if (xAccTextBox.InvokeRequired)
					this.Invoke(updateResetEvent);
				else {
					if (_data == null)
						_data = new SystemData();
					else {
						_data.Platform.Data.setEncoderCounts(0);
						_data.Platform.resetPlatformController();
						_data.Navigation.resetNavigationController();
					}
					if (_data.RaceMode)
						_strategy = new RaceStrategy(ref _data
							, double.Parse(InitializationParameters
							.IniFileParameters
							["steeringKp"])
							, double.Parse(InitializationParameters
							.IniFileParameters
							["coneDistanceThreshold"])
							, double.Parse(InitializationParameters
							.IniFileParameters
							["dummyDistanceThreshold"])
							, int.Parse(InitializationParameters
							.IniFileParameters
							["minimumSpeed"])
							, int.Parse(InitializationParameters
							.IniFileParameters
							["maximumSpeed"]));
					mapPictureBox.Image = new Bitmap(_data.MapFileName);
				}
			} catch (Exception e) {
				_data.logError(e);
			}
		}

        private void saveToolStripMenuItem_Click(object sender, EventArgs e) {
            _data.saveMap();
        }

		private void openToolStripMenuItem_Click(object sender, EventArgs e) {

		}

    }
}
