﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GpsModule;
using NavigationController;
using PlatformController;
using VisionSystem;
using UtmConvert;
using System.Drawing;

namespace system_controller {
    class SystemData {

        string _waypointsFileName = "";
        public string WaypointsFileName {
            get { return _waypointsFileName; }
        }

        string _mapFileName = "";
        public string MapFileName {
            get { return _mapFileName; }
        }

        PlatformStrategy _platform;
		public PlatformStrategy Platform {
            get { return _platform; }
            //set { _platform = value; }
        }

        NavigationStrategy _navigation;
		public NavigationStrategy Navigation {
            get { return _navigation; }
            //set { _navigation = value; }
        }
        VisionSystemMainForm _vision;

        public VisionSystemMainForm Vision {
            get { return _vision; }
            //set { _vision = value; }
        }

		GpsStrategy _gps;
		public GpsStrategy Gps {
            get { return _gps; }
            //set { _gps = value; }
        }

        UtmConvertMainForm _utm;
        public UtmConvertMainForm Utm {
            get { return _utm; }
            //set { _utm = value; }
        }

		ConvertLatLonUtm _convertUtm;
		public ConvertLatLonUtm ConvertUtm {
			get { return _convertUtm; }
		}

        System.Drawing.Point _click1, _click2;

        public System.Drawing.Point Click2 {
            get { return _click2; }
            set { _click2 = value; }
        }

        public System.Drawing.Point Click1 {
            get { return _click1; }
            set { _click1 = value; }
        }
        UtmPoint _utmPoint1, _utmPoint2;

        public UtmPoint UtmPoint1 {
            get { return _utmPoint1; }
            set { _utmPoint1 = value; }
        }

        public UtmPoint UtmPoint2 {
            get { return _utmPoint2; }
            set { _utmPoint2 = value; }
        }

        string _point1 = "", _point2 = "";
        public string Point1 {
            get { return _point1; }
            set { _point1 = value; }
        }

        public string Point2 {
            get { return _point2; }
            set { _point2 = value; }
        }

        XmlParser _parser;

        public XmlParser Parser {
            get { return _parser; }
            set { _parser = value; }
        }

        float _scaleDeltaX;
        public float ScaleDeltaX {
            get { return _scaleDeltaX; }
        }

        float _scaleDeltaY;
        public float ScaleDeltaY {
            get { return _scaleDeltaY; }
        }

        float _scaleX;
        public float ScaleX {
            get { return _scaleX; }
        }

        float _scaleY;
        public float ScaleY {
            get { return _scaleY; }
        }

		bool _raceMode = false;
		public bool RaceMode {
			get { return _raceMode; }
		}
		ErrorLogger _errorLog;

        public SystemData() {
			//need to call here because the log file and directory 
			//are stored in the ini file!
			InitializationParameters.readIniParameters();
			_errorLog = new ErrorLogger();
			_errorLog.openFile();
			InitializationParameters.updateErrorEvent += logError;
			InitializationParameters.readIniParameters();
			_convertUtm = new ConvertLatLonUtm();
            _mapFileName = InitializationParameters
                .IniFileParameters["map"];
            _waypointsFileName = InitializationParameters
                .IniFileParameters["waypoints"];
			_raceMode = InitializationParameters
				.IniFileParameters["raceMode"] == "yes";
            _parser = new XmlParser();
			//assign error events to the logData function
			_parser.updateErrorEvent += logError;
            _parser.readFile(_waypointsFileName);
            _parser.Parse("ddd_dd");
            _click1 = new System.Drawing.Point();
            _click2 = new System.Drawing.Point();
            _utmPoint1 = new UtmPoint();
            _utmPoint2 = new UtmPoint();


        }

		public void logError(Exception error) {
			_errorLog.write(DateTime.Now.ToString() + ":\r\n"
				+ "Message: " + error.Message + "\r\n"
				+ "Stack Trace: " + error.StackTrace + "\r\n"
				+ "Source: " + error.Source + "\r\n");
		}

        public void saveMap() {
            Printer p = new Printer();
            p.writeUtmXmlFile(_parser.UtmPointSetList);
        }

        public bool waypointMatch(UtmPoint p) {
            if (_point2 == "")
                return _point1 == p.Handle;
            else return _point2 == p.Handle;
        }

        public void drawRedLines(ref Bitmap map) {
            float deltaClickX = (float)_click2.X - (float)_click1.X;
            float deltaClickY = (float)_click2.Y - (float)_click1.Y;
            float deltaUtmX = (float)(_utmPoint2.Point.X - _utmPoint1.Point.X);
            float deltaUtmY = (float)(_utmPoint2.Point.Y - _utmPoint1.Point.Y);
            _scaleDeltaX = deltaClickX / deltaUtmX;
            _scaleDeltaY = deltaClickY / deltaUtmY;
            _scaleX = (float)_click1.X / (float)_utmPoint1.Point.X;
            _scaleY = (float)_click1.Y / (float)_utmPoint1.Point.Y;
            Graphics g = Graphics.FromImage(map);
            Pen pen = new Pen(Color.Red, 8);
            SolidBrush brush = new SolidBrush(Color.Orange);
            SolidBrush gBrush = new SolidBrush(Color.Gray);
            float nextX = 0, nextY = 0;
            for (int i = 0; i < _parser.UtmPointSetList[0].Points.Count; ++i) {
                float deltaX = (float)(_parser.UtmPointSetList[0].Points[i].Point.X)
                    - (float)(_utmPoint1.Point.X);
                float deltaY = (float)(_parser.UtmPointSetList[0].Points[i].Point.Y)
                    - (float)(_utmPoint1.Point.Y);
                float x1 = (float)(_parser.UtmPointSetList[0].Points[i].Point.X);
                float y1 = (float)(_parser.UtmPointSetList[0].Points[i].Point.Y);
                if (_parser.UtmPointSetList[0].Points[i].Note == "dummy")
                    g.FillEllipse(gBrush, x1 * _scaleX + deltaX * _scaleDeltaX
                        - 16, y1 * _scaleY + deltaY * _scaleDeltaY - 16, 32, 32);
                else
                    g.FillEllipse(brush, x1 * _scaleX + deltaX * _scaleDeltaX
                        - 16, y1 * _scaleY + deltaY * _scaleDeltaY - 16, 32, 32);
                g.DrawLine(pen, x1 * _scaleX + nextX, y1 * _scaleY + nextY
                    , x1 * _scaleX + deltaX * _scaleDeltaX
                    , y1 * _scaleY + deltaY * _scaleDeltaY);
                nextX = _scaleX + deltaX * _scaleDeltaX;
                nextY = _scaleY + deltaY * _scaleDeltaY;
            }
        }

		public void drawCurrentPosition(ref Bitmap map, UtmPoint point) {
			float deltaClickX = (float)_click2.X - (float)_click1.X;
			float deltaClickY = (float)_click2.Y - (float)_click1.Y;
			float deltaUtmX = (float)(_utmPoint2.Point.X - _utmPoint1.Point.X);
			float deltaUtmY = (float)(_utmPoint2.Point.Y - _utmPoint1.Point.Y);
			_scaleDeltaX = deltaClickX / deltaUtmX;
			_scaleDeltaY = deltaClickY / deltaUtmY;
			_scaleX = (float)_click1.X / (float)_utmPoint1.Point.X;
			_scaleY = (float)_click1.Y / (float)_utmPoint1.Point.Y;
			Graphics g = Graphics.FromImage(map);
			SolidBrush brush = new SolidBrush(Color.Yellow);
			Pen pen = new Pen(Color.Red, 4);
			float x1 = (float)point.Point.X;
			float y1 = (float)point.Point.Y;
			float deltaX = (float)point.Point.X - (float)(_utmPoint1.Point.X);
			float deltaY = (float)point.Point.Y - (float)(_utmPoint1.Point.Y);
			x1 = x1 * _scaleX + deltaX * _scaleDeltaX - 16;
			y1 = y1 * _scaleY + deltaY * _scaleDeltaY - 16;
			g.FillEllipse(brush, x1, y1, 32, 32);
			g.DrawLine(pen, x1 + 16, 0, x1 + 16, map.Height);
			g.DrawLine(pen, 0, y1 + 16, map.Width, y1 + 16);
		}

        public void drawClickedWaypoint(ref Bitmap map, int x, int y) {
            if (_click1.IsEmpty) { _click1.X = x; _click1.Y = y; }
            else { _click2.X = x; _click2.Y = y; } 
            Graphics g = Graphics.FromImage(map);
            SolidBrush brush = new SolidBrush(Color.Green);
            g.FillEllipse(brush, x - 16, y - 16, 32, 32);
        }

        ~SystemData() {
			safeShutdown();
        }

		public void showPlatform() {
			initPlatform();
			PlatformControllerMainForm p = new PlatformControllerMainForm(ref _platform);
			p.Show();
		}

		public void initPlatform() {
			try {
				if (_platform == null) {
					_platform = new PlatformStrategy(
						InitializationParameters
						.IniFileParameters
						["platformControllerPortName"],
						InitializationParameters
						.IniFileParameters
						["javelinIdePortName"],
						InitializationParameters
						.IniFileParameters
						["encoderCounterPortName"],
						double.Parse(InitializationParameters
						.IniFileParameters
						["countsPerMeter"]),
						double.Parse(InitializationParameters
						.IniFileParameters
						["throttleKp"]),
						double.Parse(InitializationParameters
						.IniFileParameters
						["throttleKi"]),
						double.Parse(InitializationParameters
						.IniFileParameters
						["throttleKd"]));
					_platform.updateErrorEvent += logError;
				}
			} catch (Exception e) {
				logError(e);
			}
		}

        public void showNavigation() {
			initNavigation();
			NavigationControllerMainForm p = new NavigationControllerMainForm(ref _navigation);
			p.Show();
        }

		public void initNavigation() {
			try {
				if (_navigation == null) {
					_navigation = new NavigationStrategy(
						InitializationParameters.IniFileParameters
						["navigationControllerPortName"],
						double.Parse(InitializationParameters
						.IniFileParameters["magneticDeclination"]));
					_navigation.updateErrorEvent += logError;
				}
			} catch (Exception e) {
				logError(e);
			}
		}

        public void showVision() {
            if (_vision != null && !_vision.IsDisposed) {
                _vision.Show();
            } else {
				_vision = new VisionSystemMainForm(
					int.Parse(InitializationParameters
					.IniFileParameters["red"]),
					int.Parse(InitializationParameters
					.IniFileParameters["green"]),
					int.Parse(InitializationParameters
					.IniFileParameters["blue"]));
				_vision.updateErrorEvent += logError;	
                _vision.Show();
            }
        }

		public void initVision() {
			if (_vision == null) {
				_vision = new VisionSystemMainForm(
					int.Parse(InitializationParameters
					.IniFileParameters["red"]),
					int.Parse(InitializationParameters
					.IniFileParameters["green"]),
					int.Parse(InitializationParameters
					.IniFileParameters["blue"]));
				_vision.updateErrorEvent += logError;	
			}
		}

		public void initGps() {
			try {
				if (_gps == null) {
					_gps = new GpsStrategy(
						InitializationParameters
						.IniFileParameters["gpsModulePortName"]);
					_gps.updateErrorEvent += logError;
				}
			} catch (Exception e) {
				logError(e);
			}
		}

		public void showGps() {
			initGps();
			GpsModuleMainForm g = new GpsModuleMainForm(ref _gps);
			g.Show();
		}

        public void showUtm() {
            if (_utm != null && !_utm.IsDisposed) {
				_utm.Show();
            } else {
                _utm = new UtmConvertMainForm();
                _utm.Show();
            }
        }

		public void initUtm() {
			if (_utm == null)
				_utm = new UtmConvertMainForm();
		}

		public void safeShutdown() {
			//if (_vision != null) {
			//    _vision.Silent = true;
			//    _vision.stopCamera();
			//    _vision.Close();
			//}		
			if (_gps != null) {
				_gps.setSilentMode(true);
				_gps.closePort();
			}
			if (_platform != null) {
				_platform.setSilentMode(true);
				_platform.closePort();
			}
			if (_navigation != null) {
				_navigation.setSilentMode(true);
				_navigation.closePort();
			}
		}
    }
}
